#include <string.h>
#include <stdio.h>
#include "common.h"

float mass[NUM_BODIES];  // Mass
VEC3D pos[NUM_BODIES];   // Position vectors
VEC3D vel[NUM_BODIES];   // Velocity vectors

// Performs one step of the simulation
void
step()
{
  VEC3D acc[NUM_BODIES];  // Acceleration vectors

  // Clear acceleration vectors
  memset(acc, 0, sizeof(acc));

  // Calculate acceleration of each body
  for (int i = 0; i < NUM_BODIES - 1; i++) {
    for (int j = i + 1; j < NUM_BODIES; j++) {
      VEC3D d;
      float t;

      // Calculate displacement from i to j
      d.x = pos[j].x - pos[i].x;
      d.y = pos[j].y - pos[i].y;
      d.z = pos[j].z - pos[i].z;

      // Calculate 1 / dist^2
      t = 1 / (fsqrf(d.x) + fsqrf(d.y) + fsqrf(d.z));

      // Calculate components of t along the displacement vector
      t *= (float)sqrt(t);
      d.x *= t;
      d.y *= t;
      d.z *= t;

      // Update acceleration of i 
      acc[i].x += d.x * mass[j];
      acc[i].y += d.y * mass[j];
      acc[i].z += d.z * mass[j];

      // Update acceleration of j
      acc[j].x += -d.x * mass[i];
      acc[j].y += -d.y * mass[i];
      acc[j].z += -d.z * mass[i];
    }
  }

  // Update positions and velocities
  for (int i = 0; i < NUM_BODIES; i++) {
    pos[i].x += vel[i].x * TIMESTEP;
    pos[i].y += vel[i].y * TIMESTEP;
    pos[i].z += vel[i].z * TIMESTEP;

    vel[i].x += acc[i].x * TIMESTEP;
    vel[i].y += acc[i].y * TIMESTEP;
    vel[i].z += acc[i].z * TIMESTEP;
  }
}

extern uint32_t ticks();
extern float frand();
extern void init_misc();

// Generates random mass, initial position, initial velocity parameters for all
// bodies
void
init()
{
  for (int i = 0; i < NUM_BODIES; i++) {
    mass[i] = frand() * 5000.f;
    pos[i].x = (frand() - 0.5f) * 1000;
    pos[i].y = (frand() - 0.5f) * 1000;
    pos[i].z = (frand() - 0.5f) * 1000;
    vel[i].x = (frand() - 0.5f) * 10;
    vel[i].y = (frand() - 0.5f) * 10;
    vel[i].z = (frand() - 0.5f) * 10;
  }
}

int
main()
{
  // Initialize timing and RNG
  init_misc();
  // Generate random bodies
  init();

  while (TRUE) {
    uint32_t t = ticks();

    // Perform simulation step
    step();

    printf("%d ms\n", ticks() - t);
  }

  return 0;
}
